# -----------------------------------------------------------------------------
# (C) British Crown Copyright 2017-2021 Met Office.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
#
# Plot Fig 2b from "An inter-comparison of the mass budget of the Arctic sea ice in 
# CMIP6 models", Keen et al (2021).
# XX denotes local pathnames/files etc required 


import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import read_budgetfiles as rb
import budget_utils as bu
from SIMIP_Massbudget_DataDic import DataDic # XX Dictionary defining all the filenames etc
from SIMIP_Obs_DataDic import ObsDic # XX Dictionary defining all the filenames etc
import sys

# Input data path
#-----------------
path=' ' # XX
obspath=' ' # XX


# All CMIP6 models - ordered by sea ice component
ModelNames_CMIP6_neworder = ['NCAR_CESM2_CAM',   # CICE
          'NCAR_CESM2_WACCM', 		# CICE
          'NCC_NorESM2-LM',       	# CICE
          'NCC_NorESM2-MM',       	# CICE
          'MetO_UKESM1',      		# CICE (GSI8.1)
          'MetO_GC3.1_ORCA1',  		# CICE (GSI8.1)    
          'MetO_GC3.1_ORCA025',  	# CICE (GSI8.1)
          'CSIRO_ACCESS-CM2', 		# CICE (GSI8.1)
          'UCLouvain_EC-Earth3-Veg',    # LIM3
          'UCLouvain_EC-Earth3',        # LIM3
          'IPSL_CM6A-LR',          	# LIM3
          'CCCma_CanESM5_x1',          	# LIM2
          'GFDL-CM4',         		# SIS2
          'GFDL-ESM4',        		# SIS2
          'MRI_ESM2.0']         	# MRI

ModelNames=ModelNames_CMIP6_neworder
plottag='CMIP6_models'

nskip=4
plotfile='Plot_files/Fig2b_icemass_evolution.png' 
plotfile_pdf='Plot_files/Fig2b_icemass_evolution.pdf' 


# Read in the obs
# -----------------

Obs_tag='PIOMAS_1990-2009'
filepath=obspath+ObsDic[Obs_tag]['dir']+ObsDic[Obs_tag]['file']
years_PIOMAS,months,PIOMAS=rb.read_obsfile(filepath,nskip)
PIOMAS=PIOMAS/1.0e15 # Convert from Kg to Gt x10^3
#print(years_PIOMAS)

Obs_tag2='PIOMAS_1979-2018'
#filepath=obspath+ObsDic[Obs_tag]['dir']+ObsDic[Obs_tag]['file']
filepath=obspath+' ' # XX
years_PIOMAS2,months,PIOMAS2=rb.read_obsfile(filepath,nskip)
PIOMAS2=PIOMAS2/1.0e15 # Convert from Kg to Gt x10^3

rho_PIOMAS=917.0 # kg m-3
PIOMAS2_err=2.25e3 # Conservative estimate of uncertainty in March ice volume (km^3), from Schweiger et al (2011)
PIOMAS2_err=PIOMAS2_err*rho_PIOMAS*1.0e9 # Convert from km^3 to kg
PIOMAS2_err=PIOMAS2_err/1.0e15 # Convert from Kg to Gt x10^3

# Set up arrays to hold model data for printing out...
nmodels=len(ModelNames)
March_evolution=np.empty([nmodels,140],dtype='float')

# -----------------------------------------------------------------------------
matplotlib.rcParams.update({'font.size':10})
#fig2 = plt.figure(figsize = (16,14)) 
#fig2 = plt.figure(figsize = (16,10)) # Landscape
#fig2 = plt.figure(figsize = (10,16)) # Portrait

# March ice mass ---------------------------------------------------------------
fig2 = plt.figure(figsize = (8,5)) # Landscape
ax2 = fig2.add_subplot(111) 
ax2.set_title('(b)')
ax2.set_ylabel(r'Mass (Gt x$10^3$)')
ax2.set_xlabel('Year')
ax2.plot([1960,2100],[0.0,0.0],color="black")

for n,model in enumerate(ModelNames):

	filepath=path+DataDic[model]['dir']
	Histfiles=DataDic[model]['HistFiles_ice']
	Scenariofiles=DataDic[model]['ScenarioFiles_ice']

	nhistfiles=len(Histfiles)
	nscenariofiles=DataDic[model]['nScenarioFiles_ice']

	years,icearea_mn,icearea_stdev,icemass_mn,icemass_stdev,massbudget_mn,massbudget_stdev = bu.monthly_mn_evolution(
			filepath=filepath,histfiles=Histfiles,nhistfiles=nhistfiles,
			scenariofiles=Scenariofiles,nscenariofiles=nscenariofiles,nskip=nskip,month=3)
	
	# Plotting ----------------------------------------
	
	mean=icemass_mn/1.0e15 # Convert from Kg to Gt x10^3
	stdev=icemass_stdev/1.0e15 # Convert from Kg to Gt x10^3
	ax2.plot(years,mean, 
            color=DataDic[model]['line_colour2'], 
            linestyle='-',	            
            label=DataDic[model]['label'],
            linewidth=2)
	ax2.fill_between(years, mean-stdev, mean+stdev,
		color=DataDic[model]['line_colour2'],
		alpha=0.25,
		linestyle='-',
		linewidth=0.5,
		antialiased=True)

	March_evolution[n,:]=mean[0:140]

# Add the obs ----------------------------------------
#ax2.plot(years_PIOMAS,PIOMAS[:,2], 
#color='black', 
#linestyle='-',	            
#label='PIOMAS',
#linewidth=3)

ax2.plot(years_PIOMAS2,PIOMAS2[:,2], 
color='black', 
linestyle='-',	            
label='PIOMAS',
linewidth=3)
mean=PIOMAS2[:,2]
stdev=PIOMAS2_err
ax2.fill_between(years_PIOMAS2, mean-stdev, mean+stdev,
		color='black',
		alpha=0.25,
		linestyle='-',
		linewidth=0.5,
		antialiased=True)


ax2.set_xlim([1960, 2100])
ax2.legend(loc="best", fontsize = 'x-small')    

fig2.savefig(plotfile_pdf, dpi=300, bbox_inches="tight", pad_inches=0)
plt.show()
plt.close()

# -----------------------------------------------------------

# ---------------That's all folks --------------------


